/*---------------------------------------------------------------------------*/
/* cam_tcm8240md.c                                                           */
/*                                              2010.08.23  Kyosuke Ishikawa */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Includes                                                                  */
/*---------------------------------------------------------------------------*/

#include <GenericTypeDefs.h>
#include <plib.h>

#include "cam_tcm8240md.h"

/*---------------------------------------------------------------------------*/
/* Macro Definitions                                                         */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* Function Prototypes                                                       */
/*---------------------------------------------------------------------------*/

static void cam_i2c_send_start(unsigned char);
static void cam_i2c_read_start(unsigned char);
static void cam_i2c_send(unsigned char);
static unsigned char cam_i2c_read(void);
static void cam_i2c_end(void);

/*---------------------------------------------------------------------------*/
/* Global Variables                                                          */
/*---------------------------------------------------------------------------*/

unsigned char cam_config[][2] = {
	{0x02, 0x00},	// Set Camera Active
	{0x02, 0x40},	// Set Camera Reset
	{0x03, 0x00},	// PLL
	{0x02, 0x00},	// Set Camera Active
	{0x0B, 0x00},	// White Line OFF 
	{0x58, 0x20},	// Exposure Time 
	{0x05, 0x80},	// Frame Rate Quarter(00)/Half(40)/Full(80)
	{0x1A, 0xFF},	// HCOUNT = 0x3FF = 1023 
	{0x1B, 0xB3},	// VCOUNT = 0x21B = 539
	{0x1C, 0xA1},	// 
	{0x11, 0x4A},	// Changed b/c of PICSIZE 
	{0x14, 0x23},	// Changed b/c of PICSIZE 
	{0x04, 0x0B},	// RGB  160x120  OUT ON 
	{0x1F, 0x0C},	// SPCOUNT = 0xBC3 = 3011 // Doesn't match formula? 
	{0x1E, 0x23},	// SPCOUNT[7:0] 
	{0x0E, 0x00},	// According to the AppNote it should be 0xAC for this PICSIZE?
};

const unsigned char	configAddress = 0x3D;
const unsigned char	cameraAddress = 0x3E;


/*---------------------------------------------------------------------------*/
/* cam_init                                                                  */
/*---------------------------------------------------------------------------*/

signed char cam_init(void)
{
	unsigned short	i;
	UINT32	actualClock;
	
	actualClock = I2CSetFrequency(I2C2, 80000000, 1000000);
	I2CEnable(I2C2, TRUE);
	
	for ( i = 0 ; i < sizeof(cam_config)/sizeof(cam_config[0]) ; i++ )	{
		cam_i2c_send_start(configAddress);
		cam_i2c_send(cam_config[i][0]);
		cam_i2c_send(cam_config[i][1]);
		cam_i2c_end();
	}
	
	return 0;
}

/*---------------------------------------------------------------------------*/
/* cam_get_image                                                             */
/*---------------------------------------------------------------------------*/

unsigned short cam_get_image(unsigned char *dat)
{
	unsigned char	d;
	unsigned char	c[4];
	unsigned long	count, count_n;
	
	cam_i2c_read_start(cameraAddress);
	c[0] = cam_i2c_read();
	c[1] = cam_i2c_read();
	c[2] = cam_i2c_read();
	c[3] = cam_i2c_read();
	count = c[0] | c[1] << 8 | c[2] << 16 | c[3] << 24;
	count_n = count;
	count = 160*120*2;
	
	for ( ; count > 0 ; count-- )	{
		d = cam_i2c_read();
		*dat = d;
		dat++;
	}
	cam_i2c_end();
	
	return (unsigned short)count_n;
}

/*---------------------------------------------------------------------------*/
/* cam_image_to_rgb                                                          */
/*---------------------------------------------------------------------------*/

void cam_image_to_rgb(unsigned char *dat, unsigned char *rgb, unsigned short x_size, unsigned short y_size)
{
	unsigned short	x, y;
	unsigned char	r, g, b;
	
	for ( y = 0 ; y < y_size ; y++ )	{
		for ( x = 0 ; x < x_size ; x++ )	{
			r = (*dat & 0x1F) << 3;
			g = ((*dat & 0xE0) >> 3) | ((*(dat+1) & 0x07) << 5);
			b = (*(dat+1) & 0xF8);
			rgb[(x+y*x_size)*3+0] = r;
			rgb[(x+y*x_size)*3+1] = g;
			rgb[(x+y*x_size)*3+2] = b;
			dat++;
		}
	}
}

/*---------------------------------------------------------------------------*/
/* cam_i2c_send_start                                                        */
/*---------------------------------------------------------------------------*/

static void cam_i2c_send_start(unsigned char adr)
{
	I2C_STATUS	status;
	
	while( !I2CBusIsIdle(I2C2) );
	
	if ( I2CStart(I2C2) != I2C_SUCCESS )	{
		return;
	}
	
	do	{
		status = I2CGetStatus(I2C2);
	} while ( !(status & I2C_START) );
	
	
	while ( !I2CTransmitterIsReady(I2C2) );
	
	if ( I2CSendByte(I2C2, adr<<1) == I2C_MASTER_BUS_COLLISION )	{
		return;
	}
	while ( !I2CTransmissionHasCompleted(I2C2) );
}

/*---------------------------------------------------------------------------*/
/* cam_i2c_read_start                                                        */
/*---------------------------------------------------------------------------*/

static void cam_i2c_read_start(unsigned char adr)
{
	I2C_STATUS	status;
	
	while( !I2CBusIsIdle(I2C2) );
	
	if ( I2CStart(I2C2) != I2C_SUCCESS )	{
		return;
	}
	
	do	{
		status = I2CGetStatus(I2C2);
	} while ( !(status & I2C_START) );
	
	
	while ( !I2CTransmitterIsReady(I2C2) );
	
	if ( I2CSendByte(I2C2, (adr<<1)|1) == I2C_MASTER_BUS_COLLISION )	{
		return;
	}
	while ( !I2CTransmissionHasCompleted(I2C2) );
}

/*---------------------------------------------------------------------------*/
/* cam_i2c_send                                                              */
/*---------------------------------------------------------------------------*/

static void cam_i2c_send(unsigned char d)
{
	while ( !I2CTransmitterIsReady(I2C2) );
	
	if ( I2CSendByte(I2C2, d) == I2C_MASTER_BUS_COLLISION )	{
		return;
	}
	while ( !I2CTransmissionHasCompleted(I2C2) );
}

/*---------------------------------------------------------------------------*/
/* cam_i2c_read                                                              */
/*---------------------------------------------------------------------------*/

static unsigned char cam_i2c_read(void)
{
	unsigned char	d;
	
	if ( I2CReceiverEnable(I2C2, TRUE) == I2C_RECEIVE_OVERFLOW )	{
	} else	{
		while( !I2CReceivedDataIsAvailable(I2C2) );
		I2CAcknowledgeByte(I2C2, TRUE);
		while( I2C2CON & 0x10 );
		d = I2CGetByte(I2C2);
	}
	
	return d;
}

/*---------------------------------------------------------------------------*/
/* cam_i2c_end                                                               */
/*---------------------------------------------------------------------------*/

static void cam_i2c_end(void)
{
	I2C_STATUS	status;
	
	I2CStop(I2C2);
	
	do	{
		status = I2CGetStatus(I2C2);
	} while ( !(status & I2C_STOP) );
}

/*---------------------------------------------------------------------------*/
