/*---------------------------------------------------------------------------*/
/* oled_4d.c                                                                 */
/*                                              2010.07.26  Kyosuke Ishikawa */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Includes                                                                  */
/*---------------------------------------------------------------------------*/

#include <GenericTypeDefs.h>
#include <plib.h>

#include "oled_4d.h"
#include "timer.h"
#include "uart.h"

/*---------------------------------------------------------------------------*/
/* Macro Definitions                                                         */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Function Prototypes                                                       */
/*---------------------------------------------------------------------------*/

static void oled_uart_put(unsigned char);
static signed char oled_uart_get(unsigned char *);

/*---------------------------------------------------------------------------*/
/* Global Variables                                                          */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* oled_init                                                                 */
/*---------------------------------------------------------------------------*/

void oled_init(void)
{
	unsigned char	r;
	
	// Reset
	LATEbits.LATE1 = 0;			// Reset On
	wai_1ms(10);
	
	LATEbits.LATE1 = 1;			// Reset Off
	wai_100ms(5);
	
	// Send auto baudrate command
	r = 0x00;
	do	{
		oled_uart_put(0x55);
		while ( oled_uart_get(&r) == -1 );
	} while ( r != 0x06 );
	
	// Set background color
	r = 0x00;
	do	{
		oled_uart_put(0x42);
		oled_uart_put(0x00);
		oled_uart_put(0x00);
		while ( oled_uart_get(&r) == -1 );
	} while ( r != 0x06 );

	// Clear display
	r = 0x00;
	do	{
		oled_uart_put(0x45);
		while ( oled_uart_get(&r) == -1 );
	} while ( r != 0x06 );
}

/*---------------------------------------------------------------------------*/
/* oled_pset                                                                  */
/*---------------------------------------------------------------------------*/

void oled_pset(unsigned char x, unsigned char y, unsigned short color)
{
	
}

/*---------------------------------------------------------------------------*/
/* oled_put_text                                                             */
/*---------------------------------------------------------------------------*/

void oled_put_text(unsigned char x, unsigned char y, unsigned short color, unsigned char *c)
{
	unsigned char	r;
		
	oled_uart_put(0x73);
	oled_uart_put(x);			// Position
	oled_uart_put(y);
	oled_uart_put(0);			// Font : 5x7
	oled_uart_put(color>>8);		// Color
	oled_uart_put(color & 0xFF);
	while ( *c != 0 )	{
		oled_uart_put(*c);
		c++;
	}
	oled_uart_put(0x00);
	while ( oled_uart_get(&r) == -1 );
}

/*---------------------------------------------------------------------------*/
/* oled_put_dec                                                              */
/*---------------------------------------------------------------------------*/

void oled_put_dec(unsigned char x, unsigned char y, unsigned short color, unsigned short d)
{
	unsigned short	dd;
	unsigned short	n;
	
	do	{
		n = 1;
		for ( dd = d ; dd > 9 ; )	{
			dd /= 10;
			n *= 10;
		}
		oled_put_hex( x, y, color, (unsigned char)dd);
		x++;
		d = d - dd * n;
	} while ( d > 9 );
	oled_put_hex( x, y, color, (unsigned char)d);
}

/*---------------------------------------------------------------------------*/
/* oled_put_hex                                                              */
/*---------------------------------------------------------------------------*/

void oled_put_hex(unsigned char x, unsigned char y, unsigned short color, unsigned char d)
{
	unsigned char	r;
	
	oled_uart_put(0x73);
	oled_uart_put(x);			// Position
	oled_uart_put(y);
	oled_uart_put(0);			// Font : 5x7
	oled_uart_put(color>>8);		// Color
	oled_uart_put(color & 0xFF);
	d = d & 0x0F;
	if ( d > 9 )	{
		d = d + 'A' - 10;
	} else	{
		d = d + '0';
	}
	oled_uart_put(d);
	oled_uart_put(0x00);
	while ( oled_uart_get(&r) == -1 );
}

/*---------------------------------------------------------------------------*/
/* oled_put_hex2                                                             */
/*---------------------------------------------------------------------------*/

void oled_put_hex2(unsigned char x, unsigned char y, unsigned short color, unsigned char d)
{
	oled_put_hex( x,   y, color, (d>>4) & 0x0F );
	oled_put_hex( x+1, y, color, d & 0x0F );
}

/*---------------------------------------------------------------------------*/
/* oled_put_hex4                                                             */
/*---------------------------------------------------------------------------*/

void oled_put_hex4(unsigned char x, unsigned char y, unsigned short color, unsigned short d)
{
	oled_put_hex2( x,   y, color, (unsigned char)( (d>>8) & 0xFF ) );
	oled_put_hex2( x+2, y, color, (unsigned char)( d & 0xFF ) );
}

/*---------------------------------------------------------------------------*/
/* oled_draw_triangle                                                        */
/*---------------------------------------------------------------------------*/

void oled_draw_triangle(unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2, unsigned char x3, unsigned char y3, unsigned short color, unsigned char p)
{
	unsigned char	r;
	
	oled_uart_put(0x70);
	oled_uart_put(p);
	while ( oled_uart_get(&r) == -1 );
	oled_uart_put(0x47);
	oled_uart_put(x1);
	oled_uart_put(y1);
	oled_uart_put(x2);
	oled_uart_put(y2);
	oled_uart_put(x3);
	oled_uart_put(y3);
	oled_uart_put(color>>8);		// Color
	oled_uart_put(color & 0xFF);
	while ( oled_uart_get(&r) == -1 );
}

/*---------------------------------------------------------------------------*/
/* oled_draw_image16                                                         */
/*---------------------------------------------------------------------------*/

void oled_draw_image16(unsigned char x, unsigned char y, unsigned char width, unsigned char height, unsigned char *d)
{
	unsigned char	r, g, b, c;
	unsigned short	i;
	
	oled_uart_put(0x49);
	oled_uart_put(x);
	oled_uart_put(y);
	oled_uart_put(width);
	oled_uart_put(height);
	oled_uart_put(16);
	for ( i=0 ; i<width*height ; i++ )	{
		r = *d++;
		g = *d++;
		b = *d++;
		c = (r << 4) | ((g & 0x0E ) >> 1);
		oled_uart_put(c);
		c = ((g & 0x01) << 7) | (b << 1);
		oled_uart_put(c);
	}
	while ( oled_uart_get(&r) == -1 );
}

/*---------------------------------------------------------------------------*/
/* oled_draw_image2                                                         */
/*---------------------------------------------------------------------------*/

void oled_draw_image2(unsigned char x, unsigned char y, unsigned char width, unsigned char height, unsigned char *d)
{
	unsigned char	r;	
	unsigned short	i;
	
	oled_uart_put(0x49);
	oled_uart_put(x);
	oled_uart_put(y);
	oled_uart_put(width);
	oled_uart_put(height);
	oled_uart_put(8);
	for ( i=0 ; i<width*height ; i++ )	{
		if ( *d++ == 0x00 )	{
			oled_uart_put(0x00);
		} else	{
			oled_uart_put(0xFF);
		}
	}
	while ( oled_uart_get(&r) == -1 );
}

/*---------------------------------------------------------------------------*/
/* oled_box_fill                                                              */
/*---------------------------------------------------------------------------*/

void oled_box_fill(unsigned char x, unsigned char y, unsigned char sx, unsigned char sy, unsigned short color)
{
	unsigned char	r;
	
	// Set pen size solid
	oled_uart_put(0x70);
	oled_uart_put(0x00);
	while ( oled_uart_get(&r) == -1 );
	// Draw rectangle
	oled_uart_put(0x72);
	oled_uart_put(x);
	oled_uart_put(y);
	oled_uart_put(x+sx-1);
	oled_uart_put(y+sy-1);
	oled_uart_put((unsigned char)(color>>8));
	oled_uart_put((unsigned char)(color&0x00FF));
	while ( oled_uart_get(&r) == -1 );
}

/*---------------------------------------------------------------------------*/
/* oled_box_line                                                              */
/*---------------------------------------------------------------------------*/

void oled_box_line(unsigned char x, unsigned char y, unsigned char sx, unsigned char sy, unsigned short color)
{
	unsigned char	r;
	
	// Set pen size frame
	oled_uart_put(0x70);
	oled_uart_put(0x01);
	while ( oled_uart_get(&r) == -1 );
	// Draw rectangle
	oled_uart_put(0x72);
	oled_uart_put(x);
	oled_uart_put(y);
	oled_uart_put(x+sx-1);
	oled_uart_put(y+sy-1);
	oled_uart_put((unsigned char)(color>>8));
	oled_uart_put((unsigned char)(color&0x00FF));
	while ( oled_uart_get(&r) == -1 );
}

/*---------------------------------------------------------------------------*/
/* oled_clear                                                                */
/*---------------------------------------------------------------------------*/

void oled_clear(unsigned short c)
{
	oled_box_fill(0, 0, OLED_WIDTH, OLED_HEIGHT, c);
}

/*---------------------------------------------------------------------------*/
/* oled_uart_put                                                             */
/*---------------------------------------------------------------------------*/

static void oled_uart_put(unsigned char d)
{
	uart_put(UART3A, d);
}

/*---------------------------------------------------------------------------*/
/* oled_uart_get                                                             */
/*---------------------------------------------------------------------------*/

static signed char oled_uart_get(unsigned char *d)
{
	return uart_get(UART3A, d);
}

/*---------------------------------------------------------------------------*/
