/*---------------------------------------------------------------------------*/
/* oled_aitendo                                                              */
/*                                              2010.07.31  Kyosuke Ishikawa */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Includes                                                                  */
/*---------------------------------------------------------------------------*/

#include <GenericTypeDefs.h>
#include <plib.h>

#include "oled_aitendo.h"
#include "timer.h"
#include "spi.h"
#include "kfont8.h"

/*---------------------------------------------------------------------------*/
/* Macro Definitions                                                         */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Function Prototypes                                                       */
/*---------------------------------------------------------------------------*/

static void oled_put_1byte_code(unsigned char);
static void oled_put_2byte_code(unsigned short);

static void oled_command_write(unsigned char);
static void oled_data_write(unsigned char);
static void oled_data_write_16(unsigned short);
static void oled_spi_put(unsigned char);
static void oled_cs_on();
static void oled_cs_off();
static void oled_reset_on();
static void oled_reset_off();
static void oled_dc_command();
static void oled_dc_data();

/*---------------------------------------------------------------------------*/
/* Global Variables                                                          */
/*---------------------------------------------------------------------------*/

unsigned char	oled_2byte_code;
unsigned char	oled_cursor_x;
unsigned char	oled_cursor_y;

/*---------------------------------------------------------------------------*/
/* oled_init                                                                 */
/*---------------------------------------------------------------------------*/

void oled_init(void)
{
	oled_2byte_code = 0;
	oled_cursor_x = 0;
	oled_cursor_y = 0;
	
	// Init Port
	oled_reset_off();
	oled_cs_off();
	oled_dc_command();
	
	// Reset
	wai_1ms(200);
	oled_reset_on();
	wai_1ms(200);
	oled_reset_off();
	wai_1ms(200);
	
	// Off Display
	oled_command_write(0xAE);
	
	// Set Address
	oled_command_write(0x75);			// Set Row Address
	oled_command_write(0);
	oled_command_write(OLED_HEIGHT-1);
	oled_command_write(0x15);			// Set Column Address
	oled_command_write(0);
	oled_command_write(OLED_WIDTH-1);
	
	//
	oled_command_write(0xa0);			//Set remap & data format 0111 0000
	oled_command_write(0x74);
	oled_command_write(0xa1);			//set display star row RAM
	oled_command_write(0x00);
	oled_command_write(0xa2);			//set dispaly offset
	oled_command_write(0x00);
	oled_command_write(0xa4);			//Set Display Mode
	oled_command_write(0xa8);			//Set Multiplex Ratio
	oled_command_write(0x3f);
	oled_command_write(0xad);			//Set Master Configuration
	oled_command_write(0x8f);			//(External VCC Supply Selected)
	oled_command_write(0xB0);			//Set Power Saving Mode
	oled_command_write(0x1a);
	oled_command_write(0xB1);			//Set Phase 1 & 2 Period Adjustment
	oled_command_write(0x74);
	oled_command_write(0xb3);		 	//Set Display Clock Divide Ratio / Oscillator Frequency
	oled_command_write(0xd0);
	oled_command_write(0x8A);			//Set Second Pre-charge Speed of Color A
	oled_command_write(0x81);
	oled_command_write(0x8B);			//Set Second Pre-charge Speed of Color B
	oled_command_write(0x82);
	oled_command_write(0x8C);			//Set Second Pre-charge Speed of Color C
	oled_command_write(0x83);
	oled_command_write(0xBB);			//Set Pre-charge Level
	oled_command_write(0x3e);
	oled_command_write(0xBE);			//Set VCOMH
	oled_command_write(0x3e);
	oled_command_write(0x87);			//Set Master Current Control
	oled_command_write(0x0f);
	oled_command_write(0x81);			//Set Contrast Control for Color gAh
	oled_command_write(0x80);
	oled_command_write(0x82);			//Set Contrast Control for Color gBh
	oled_command_write(0x80);
	oled_command_write(0x83);			//Set Contrast Control for Color gCh
	oled_command_write(0x80);
	
	// On Display
	oled_command_write(0xAF);
}

/*---------------------------------------------------------------------------*/
/* oled_clear                                                                */
/*---------------------------------------------------------------------------*/

void oled_clear(unsigned short color)
{
	unsigned char	x, y;
	
	oled_command_write(0x75);			// Set Row Address
	oled_command_write(0);
	oled_command_write(OLED_HEIGHT-1);
	oled_command_write(0x15);			// Set Column Address
	oled_command_write(0);
	oled_command_write(OLED_WIDTH-1);
	for ( y=0 ; y<OLED_HEIGHT ; y++ )	{
		for ( x=0 ; x<OLED_WIDTH ; x++ )	{
			oled_data_write_16(color);
		}
	}
	oled_data_write_16(color);
}

/*---------------------------------------------------------------------------*/
/* oled_pset                                                                 */
/*---------------------------------------------------------------------------*/

void oled_pset(unsigned char x, unsigned char y, unsigned short color)
{
	oled_command_write(0x15);			// Set Column Address
	oled_command_write(x);
	oled_command_write(x);
	oled_command_write(0x75);			// Set Row Address
	oled_command_write(y);
	oled_command_write(y);
	oled_data_write_16(color);
	oled_data_write_16(color);
}

/*---------------------------------------------------------------------------*/
/* oled_locate                                                               */
/*---------------------------------------------------------------------------*/

void oled_locate(unsigned char x, unsigned char y)
{
	oled_cursor_x = x;
	oled_cursor_y = y;
}

/*---------------------------------------------------------------------------*/
/* oled_puts                                                                 */
/*---------------------------------------------------------------------------*/

void oled_puts(unsigned char *d)
{
	while ( *d != 0x00 )	{
		oled_putc(*d);
		d++;
	}
}

/*---------------------------------------------------------------------------*/
/* oled_putc                                                                 */
/*---------------------------------------------------------------------------*/

void oled_putc(unsigned char d)
{
	if ( oled_2byte_code > 0 )	{
		oled_put_2byte_code(oled_2byte_code << 8 | d);
		oled_2byte_code = 0;
	} else if ( (0x81 <= d && d <= 0x9F) || (0xE0 <= d && d <= 0xFC) )	{
        oled_2byte_code = d;
	} else	{
		oled_put_1byte_code(d);
	}
}

/*---------------------------------------------------------------------------*/
/* oled_put_1byte_code                                                       */
/*---------------------------------------------------------------------------*/

static void oled_put_1byte_code(unsigned char d)
{
	unsigned char	x, y;
	unsigned char	c;
	
	for ( x=0 ; x<4 ; x++ )	{
		c = font4[d*4+x];
		for ( y=0 ; y<8 ; y++ )	{
			if ( (c & 0x01) )	{
				oled_pset(oled_cursor_x*4+x, oled_cursor_y*8+y, OLED_COLOR_WHITE);
			}
			c = c >> 1;
		}
	}
	oled_cursor_x++;
	if ( oled_cursor_x >= OLED_WIDTH/4 )	{
		oled_cursor_x = 0;
		oled_cursor_y++;
	}
}

/*---------------------------------------------------------------------------*/
/* oled_put_2byte_code                                                       */
/*---------------------------------------------------------------------------*/

static void oled_put_2byte_code(unsigned short d)
{
	unsigned char	x, y;
	unsigned char	c;
	unsigned char	*p;
	unsigned short	i, sum;
	
	p = NULL;
	sum = 0;
	for ( i=0 ; i<92 ; i++)	{
		if ( font8table[i].start <= d && d <= font8table[i].end)	{
			p = (unsigned char *)&(font8[(sum+(unsigned short)d-font8table[i].start)<<3]);
			break;
		}
		sum += font8table[i].end - font8table[i].start + 1;
    }
	if ( p == NULL )	{
		return;
	}
	
	for ( x=0 ; x<8 ; x++ )	{
		c = p[x];
		for ( y=0 ; y<8 ; y++ )	{
			if ( (c & 0x01) )	{
				oled_pset(oled_cursor_x*4+x, oled_cursor_y*8+y, OLED_COLOR_WHITE);
			}
			c = c >> 1;
		}
	}
	oled_cursor_x += 2;
	if ( oled_cursor_x >= OLED_WIDTH/4 )	{
		oled_cursor_x = 0;
		oled_cursor_y++;
	}
}

/*---------------------------------------------------------------------------*/
/* oled_command_write                                                        */
/*---------------------------------------------------------------------------*/

static void oled_command_write(unsigned char d)
{
	oled_cs_off();
	oled_dc_command();
	oled_cs_on();
	
	oled_spi_put(d);
	
	oled_cs_off();
}

/*---------------------------------------------------------------------------*/
/* oled_data_write                                                           */
/*---------------------------------------------------------------------------*/

static void oled_data_write(unsigned char d)
{
	oled_cs_off();
	oled_dc_data();
	oled_cs_on();
	
	oled_spi_put(d);
	
	oled_cs_off();
}

/*---------------------------------------------------------------------------*/
/* oled_data_write_16                                                        */
/*---------------------------------------------------------------------------*/

static void oled_data_write_16(unsigned short d)
{
	oled_data_write( (d>>8) & 0x00FF );
	oled_data_write( d & 0x00FF );
}

/*---------------------------------------------------------------------------*/
/* oled_spi_put                                                              */
/*---------------------------------------------------------------------------*/

static void oled_spi_put(unsigned char d)
{
#if 1
	spi_put(SPI_CHANNEL1, d);
#else
	unsigned char i;
	
	for ( i=0 ; i<8 ; i++ )	{
		LATDbits.LATD10 = 0;
		if ( (d&0x80) == 0x80 )	{
			LATDbits.LATD0 = 1;
			LATDbits.LATD10 = 1;
		} else	{
			LATDbits.LATD0 = 0;
			LATDbits.LATD10 = 1;
		}
		d = d << 1;
	}
#endif
}

/*---------------------------------------------------------------------------*/
/* oled_cs_on                                                                */
/*---------------------------------------------------------------------------*/

static void oled_cs_on(void)
{
	LATEbits.LATE2 = 0;			// RE2 = L
}

/*---------------------------------------------------------------------------*/
/* oled_cs_off                                                               */
/*---------------------------------------------------------------------------*/

static void oled_cs_off(void)
{
	LATEbits.LATE2 = 1;			// RE2 = H
}

/*---------------------------------------------------------------------------*/
/* oled_reset_on                                                             */
/*---------------------------------------------------------------------------*/

static void oled_reset_on(void)
{
	LATEbits.LATE3 = 0;			// RE3 = L
}

/*---------------------------------------------------------------------------*/
/* oled_reset_off                                                            */
/*---------------------------------------------------------------------------*/

static void oled_reset_off(void)
{
	LATEbits.LATE3 = 1;			// RE3 = H
}

/*---------------------------------------------------------------------------*/
/* oled_dc_command                                                           */
/*---------------------------------------------------------------------------*/

static void oled_dc_command(void)
{
	LATEbits.LATE4 = 0;			// RE4 = L
}

/*---------------------------------------------------------------------------*/
/* oled_dc_data                                                              */
/*---------------------------------------------------------------------------*/

static void oled_dc_data(void)
{
	LATEbits.LATE4 = 1;			// RE4 = H
}

/*---------------------------------------------------------------------------*/
