/*---------------------------------------------------------------------------*/
/* oled_test.c                                                               */
/*                                              2010.08.01  Kyosuke Ishikawa */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Includes                                                                  */
/*---------------------------------------------------------------------------*/

#include <GenericTypeDefs.h>
#include <plib.h>

#include "timer.h"
#include "uart.h"
#include "spi.h"
//#include "oled_4d.h"
#include "oled_aitendo.h"

/*---------------------------------------------------------------------------*/
/* Configuration Definitions                                                 */
/*---------------------------------------------------------------------------*/

// Main clock = 16MHz/FPLLIDIV(DIV_4)*FPLLMUL(MUL_20)/FPLLODIV(DIV_1) = 80MHz
// Peripheral clock = 80MHz/FPBDIV(DIV_4)                             = 20MHz
#pragma config FPLLMUL = MUL_20, FPLLIDIV = DIV_4, FPLLODIV = DIV_1, FWDTEN = OFF
#pragma config POSCMOD = HS, FNOSC = PRIPLL, FPBDIV = DIV_4

/*---------------------------------------------------------------------------*/
/* Macro Definitions                                                         */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Function Prototypes                                                       */
/*---------------------------------------------------------------------------*/

void sys_init();

/*---------------------------------------------------------------------------*/
/* Global Variables                                                          */
/*---------------------------------------------------------------------------*/

unsigned char	dat[160*120*2];
unsigned char	rgb[160*120*3];

/*---------------------------------------------------------------------------*/
/* main                                                                      */
/*---------------------------------------------------------------------------*/

int main(void)
{
	unsigned char	x, y;
	unsigned short	*color;
	unsigned short	count;
	
	sys_init();
	timer_init();
	uart_init();
	spi_init();
	
    INTConfigureSystem(INT_SYSTEM_CONFIG_MULT_VECTOR);
    INTEnableInterrupts();
	
	oled_init();
	oled_clear(OLED_COLOR_BLACK);
//	oled_put_text(0, 1, 0xF800, (unsigned char *)"Init OK");
	cam_init();

#if 0
	oled_locate(2, 1);
	oled_puts("I͑ς炸ʖڂ");
	for ( x=0 ; x<96/2 ; x++ )	{
		for ( y=0 ; y<64/2 ; y++ )	{
//			oled_pset(x, y, OLED_COLOR_RED);
		}
	}
#endif
	
	while(1)	{
		LATEbits.LATE0 = 0;			// LED On		
		count = cam_get_image(dat);
		LATEbits.LATE0 = 1;			// LED Off
		color = (unsigned short *)dat;
		for ( y=0 ; y<120/2 ; y++ )	{
			for ( x=0 ; x<160/2 ; x++ )	{
				oled_pset(x+8, y+2, color[0]);
				color += 2;
			}
		}
//		cam_init();
#if 0
		oled_clear(OLED_COLOR_YELLOW);
		oled_clear(OLED_COLOR_BLACK);
		oled_locate(0, 0);
		oled_putc((count/10000)+'0');
		count = count % 10000;
		oled_locate(1, 0);
		oled_putc((count/1000)+'0');
		count = count % 1000;
		oled_locate(2, 0);
		oled_putc((count/100)+'0');
		count = count % 100;
		oled_locate(3, 0);
		oled_putc((count/10)+'0');
		count = count % 10;
		oled_locate(4, 0);
		oled_putc(count+'0');
#endif
	}
#if 0
	while(1)	{
		LATEbits.LATE0 = 0;			// LED On		
		uart_put(UART1A, 'A');
//		oled_clear(OLED_COLOR_RED);
		wai_100ms(5);
		
		LATEbits.LATE0 = 1;			// LED Off
		uart_put(UART1A, 'B');
//		oled_clear(OLED_COLOR_BLUE);
		wai_100ms(5);		
	}
#endif
}

/*---------------------------------------------------------------------------*/
/* sys_init                                                                  */
/*---------------------------------------------------------------------------*/

void sys_init(void)
{
// Port A
//  0 -   
//  1 -   
//  2 -   
//  3 -   
//  4 -   
//  5 -   
//  6 -   
//  7 -   
//  8 -   
//  9 -   
// 10 -   
// 11 -   
// 12 -   
// 13 -   
// 14 -   
// 15 -   

// Port B
//  0 -   
//  1 -   
//  2 -   
//  3 -   
//  4 -   
//  5 -   
//  6 -   
//  7 -   
//  8 -   
//  9 -   
// 10 -   
// 11 -   
// 12 -   
// 13 -   
// 14 -   
// 15 -   

// Port C
//  0 -   
//  1 -   
//  2 -   
//  3 -   
//  4 -   SDI1 (SPI OLED)
//  5 -   
//  6 -   
//  7 -   
//  8 -   
//  9 -   
// 10 -   
// 11 -   
// 12 -   
// 13 -   
// 14 -   
// 15 -   

// Port D
//  0 -   SDO1 (SPI OLED)
//  1 -   
//  2 -   
//  3 -   
//  4 -   
//  5 -   
//  6 -   
//  7 -   
//  8 -   
//  9 -   
// 10 -   SCK1 (SPI OLED)
// 11 -   
// 12 -   
// 13 -   
// 14 -   
// 15 -   

// Port E
//  0 OUT LED
//  1 OUT OLED /RESET
//  2 OUT SPI OLED /CS
//  3 OUT SPI OLED /RESET
//  4 OUT SPI OLED D/C
//  5 -   
//  6 -   
//  7 -   
//  8 -   
//  9 -   
// 10 -   
// 11 -   
// 12 -   
// 13 -   
// 14 -   
// 15 -   

// Port F
//  0 -   
//  1 -   
//  2 -   UART1A RX (Camera)
//  3 -   
//  4 -   UART3A RX (OLED)
//  5 -   UART3A TX (OLED)
//  6 -   
//  7 -   
//  8 -   UART1A TX (Camera)
//  9 -   
// 10 -   
// 11 -   
// 12 -   
// 13 -   
// 14 -   
// 15 -   

// Port G
//  0 -   
//  1 -   
//  2 -   
//  3 -   
//  4 -   
//  5 -   
//  6 -   
//  7 -   
//  8 -   
//  9 -   
// 10 -   
// 11 -   
// 12 -   
// 13 -   
// 14 -   
// 15 -   

	TRISEbits.TRISE0 = 0;			// RE0 as Output port
	TRISEbits.TRISE1 = 0;			// RE1 as Output port
	TRISEbits.TRISE2 = 0;			// RE2 as Output port
	TRISEbits.TRISE3 = 0;			// RE3 as Output port
	TRISEbits.TRISE4 = 0;			// RE4 as Output port
	LATEbits.LATE0 = 1;				// LED Off
	LATEbits.LATE1 = 1;				// Reset Off
	LATEbits.LATE2 = 1;				// SPI OLED CS Off
	LATEbits.LATE3 = 1;				// SPI OLED Reset Off
	LATEbits.LATE4 = 0;				// SPI OLED D/C Clear
}

/*---------------------------------------------------------------------------*/
