/*---------------------------------------------------------------------------*/
/* timer.c                                                                   */
/*                                              2010.07.24  Kyosuke Ishikawa */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Includes                                                                  */
/*---------------------------------------------------------------------------*/

#include <GenericTypeDefs.h>
#include <plib.h>

#include "timer.h"

/*---------------------------------------------------------------------------*/
/* Macro Definitions                                                         */
/*---------------------------------------------------------------------------*/

#define PERIPHERAL_CLOCK		(20000000UL)		// 20MHz
#define PRESCALE				256
#define TIMER_TICK				1					// 1ms

/*---------------------------------------------------------------------------*/
/* Function Prototypes                                                       */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Global Variables                                                          */
/*---------------------------------------------------------------------------*/

volatile unsigned short		timer_count;

/*---------------------------------------------------------------------------*/
/* timer_init                                                                */
/*---------------------------------------------------------------------------*/

void timer_init(void)
{
	timer_count = 0;
	
	OpenTimer1(T1_ON | T1_SOURCE_INT | T1_PS_1_256, (PERIPHERAL_CLOCK/PRESCALE/(1000/TIMER_TICK)));
	ConfigIntTimer1(T1_INT_ON | T1_INT_PRIOR_2);
}

/*---------------------------------------------------------------------------*/
/* Timer1Handler                                                             */
/*---------------------------------------------------------------------------*/

void __ISR(_TIMER_1_VECTOR, ipl2) Timer1Handler(void)
{
	mT1ClearIntFlag();
	
	timer_count++;
}

/*---------------------------------------------------------------------------*/
/* timer_get                                                                 */
/*---------------------------------------------------------------------------*/

unsigned short timer_get(void)
{
	return timer_count;
}

/*---------------------------------------------------------------------------*/
/* wai_1ms                                                                   */
/*---------------------------------------------------------------------------*/

void wai_1ms(unsigned char val)
{
	volatile unsigned short	t;
	
	do {
		t = timer_get();
		while ( t == timer_get() );
		val--;
	} while ( val > 0 );
}

/*---------------------------------------------------------------------------*/
/* wai_100ms                                                                 */
/*---------------------------------------------------------------------------*/

void wai_100ms(unsigned char val)
{
	do {
		wai_1ms(100);
		val--;
	} while ( val > 0 );
}

/*---------------------------------------------------------------------------*/
