/*---------------------------------------------------------------------------*/
/* uart.c                                                                    */
/*                                              2010.07.25  Kyosuke Ishikawa */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Includes                                                                  */
/*---------------------------------------------------------------------------*/

#include <GenericTypeDefs.h>
#include <plib.h>

#include "uart.h"

/*---------------------------------------------------------------------------*/
/* Macro Definitions                                                         */
/*---------------------------------------------------------------------------*/

#define BAUD_RATE_UART1A		115200				// 115.2Kbps
#define BAUD_RATE_UART3A		115200				// 115.2Kbps
#define PERIPHERAL_CLOCK		(20000000UL)		// 20MHz

/*---------------------------------------------------------------------------*/
/* Function Prototypes                                                       */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Global Variables                                                          */
/*---------------------------------------------------------------------------*/

volatile unsigned char	bufr_1a_p_w;
volatile unsigned char	bufr_1a_p_r;
unsigned char	bufr_1a[256];

volatile unsigned char	bufr_3a_p_w;
volatile unsigned char	bufr_3a_p_r;
unsigned char	bufr_3a[256];

/*---------------------------------------------------------------------------*/
/* uart_init                                                                 */
/*---------------------------------------------------------------------------*/

void uart_init(void)
{
	bufr_1a_p_w = 0;
	bufr_1a_p_r = 0;
	
	bufr_3a_p_w = 0;
	bufr_3a_p_r = 0;
	
	// Init UART1A
	UARTConfigure(UART1A, UART_ENABLE_PINS_TX_RX_ONLY);
	UARTSetFifoMode(UART1A, UART_INTERRUPT_ON_RX_NOT_EMPTY);
	UARTSetLineControl(UART1A, UART_DATA_SIZE_8_BITS | UART_PARITY_NONE | UART_STOP_BITS_1);
	UARTSetDataRate(UART1A, PERIPHERAL_CLOCK, BAUD_RATE_UART1A);
	UARTEnable(UART1A, UART_ENABLE_FLAGS(UART_PERIPHERAL | UART_RX | UART_TX));
	
	INTEnable(INT_SOURCE_UART_RX(UART1A), INT_ENABLED);
	INTSetVectorPriority(INT_VECTOR_UART(UART1A), INT_PRIORITY_LEVEL_2);
	INTSetVectorSubPriority(INT_VECTOR_UART(UART1A), INT_SUB_PRIORITY_LEVEL_0);
	
	// Init UART3A
	UARTConfigure(UART3A, UART_ENABLE_PINS_TX_RX_ONLY);
	UARTSetFifoMode(UART3A, UART_INTERRUPT_ON_RX_NOT_EMPTY);
	UARTSetLineControl(UART3A, UART_DATA_SIZE_8_BITS | UART_PARITY_NONE | UART_STOP_BITS_1);
	UARTSetDataRate(UART3A, PERIPHERAL_CLOCK, BAUD_RATE_UART3A);
	UARTEnable(UART3A, UART_ENABLE_FLAGS(UART_PERIPHERAL | UART_RX | UART_TX));
	
	INTEnable(INT_SOURCE_UART_RX(UART3A), INT_ENABLED);
	INTSetVectorPriority(INT_VECTOR_UART(UART3A), INT_PRIORITY_LEVEL_2);
	INTSetVectorSubPriority(INT_VECTOR_UART(UART3A), INT_SUB_PRIORITY_LEVEL_0);
}

/*---------------------------------------------------------------------------*/
/* IntUart1AHandler                                                          */
/*---------------------------------------------------------------------------*/

void __ISR(_UART_1A_VECTOR, ipl2) IntUart1AHandler(void)
{
	if ( INTGetFlag(INT_SOURCE_UART_RX(UART1A)) )	{
		bufr_1a[bufr_1a_p_w] = UARTGetDataByte(UART1A);
		bufr_1a_p_w++;
		INTClearFlag(INT_SOURCE_UART_RX(UART1A));
	}
	if ( INTGetFlag(INT_SOURCE_UART_TX(UART1A)) )	{
		INTClearFlag(INT_SOURCE_UART_TX(UART1A));
	}
}

/*---------------------------------------------------------------------------*/
/* IntUart3AHandler                                                          */
/*---------------------------------------------------------------------------*/

void __ISR(_UART_3A_VECTOR, ipl2) IntUart3AHandler(void)
{
	if ( INTGetFlag(INT_SOURCE_UART_RX(UART3A)) )	{
		bufr_3a[bufr_3a_p_w] = UARTGetDataByte(UART3A);
		bufr_3a_p_w++;
		INTClearFlag(INT_SOURCE_UART_RX(UART3A));
	}
	if ( INTGetFlag(INT_SOURCE_UART_TX(UART3A)) )	{
		INTClearFlag(INT_SOURCE_UART_TX(UART3A));
	}
}

/*---------------------------------------------------------------------------*/
/* uart_put                                                                  */
/*---------------------------------------------------------------------------*/

void uart_put(unsigned char ch, unsigned char d)
{
	while( !UARTTransmitterIsReady(ch) );
	UARTSendDataByte(ch, d);
}

/*---------------------------------------------------------------------------*/
/* uart_get                                                                  */
/*---------------------------------------------------------------------------*/

signed char uart_get(unsigned char ch, unsigned char *d)
{
	if ( ch == UART1A )	{
		if ( bufr_1a_p_w != bufr_1a_p_r )	{
			*d = bufr_1a[bufr_1a_p_r];
			bufr_1a_p_r++;
			return 0;
		} else	{
			return -1;
		}
	} else if ( ch == UART3A )	{
		if ( bufr_3a_p_w != bufr_3a_p_r )	{
			*d = bufr_3a[bufr_3a_p_r];
			bufr_3a_p_r++;
			return 0;
		} else	{
			return -1;
		}
	}
}

/*---------------------------------------------------------------------------*/
/* uart_put_hex                                                              */
/*---------------------------------------------------------------------------*/

void uart_put_hex(unsigned char ch, unsigned char d)
{
	d = d & 0x0F;
	if ( d > 9 )	{
		d = d + 'A' - 10;
	} else	{
		d = d + '0';
	}
	uart_put(ch, d);
}

/*---------------------------------------------------------------------------*/
/* uart_put_hex2                                                             */
/*---------------------------------------------------------------------------*/

void uart_put_hex2(unsigned char ch, unsigned char d)
{
	uart_put_hex( ch, (d>>4) & 0x0F );
	uart_put_hex( ch, d & 0x0F );
}

/*---------------------------------------------------------------------------*/
/* uart_put_hex4                                                             */
/*---------------------------------------------------------------------------*/

void uart_put_hex4(unsigned char ch, unsigned short d)
{
	uart_put_hex2( ch, (unsigned char)( (d>>8) & 0xFF ) );
	uart_put_hex2( ch, (unsigned char)( d & 0xFF ) );
}

/*---------------------------------------------------------------------------*/
/* uart_put_crlf                                                             */
/*---------------------------------------------------------------------------*/

void uart_put_crlf(unsigned char ch)
{
	uart_put(ch, 0x0A);
	uart_put(ch, 0x0D);
}

/*---------------------------------------------------------------------------*/
